/*
 *	File:		eeprom.c
 *  Project:	Oktopod_Board firmware (MCU-AtMega16)
 *	Version:	V3.2
 *  Authors:	Robert Kovacs, Milan Romic
 *	Contact:	robikovacs@oktopodstudio.com, milan.romic@oktopodstudio.com
 *	Web:		<http://www.oktopodstudio.com>

 *	This program is distributed under the terms of the GNU General Public License
 *	Copyright (c) 2010-2013 Oktopod Studio  
 *	All rights reserved.
 
 *	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

 *	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 *	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include "globals.h"
#include "eeprom.h"

uchar EEPROM_read_char(uint address, uchar *data)
{
  if (address > EEPROM_MAX_ADDR) return 1;
  while (EECR & (1<<EEWE));
  EEAR = address;
  EECR |= (1<<EERE);
  *data = EEDR;
  return 0;
}
uchar EEPROM_read_block(uint start_address, uchar *data, uint size)
{
uint count;

  if (start_address + size > EEPROM_MAX_ADDR) return 1;
  for (count=0;count<size;count++)
  {
    while (EECR & (1<<EEWE));
    EEAR = start_address + count;
    EECR |= (1<<EERE);
    *(data + count) = EEDR;
  }
  return 0;
}

uchar EEPROM_write_char(uint address, uchar data)
{
  if (address > EEPROM_MAX_ADR) return 1;
  while (EECR & (1<<EEWE));
  EEAR = address;
  EEDR = data;
  DesableAll();
  EECR |= (1<<EEMWE);
  EECR |= (1<<EEWE);
  EnableAll();
  return 0;
}

uchar EEPROM_write_block(uint start_address, uchar *data, uint size)
{
uint count;

  if (start_address + size > EEPROM_MAX_ADR) return 1;
  DesableAll();
  for (count=0;count<size;count++)
  {
    asm("wdr");
    while (EECR & (1<<EEWE));
    EEAR = start_address + count;
    EEDR = *(data + count);
    EECR |= (1<<EEMWE);
    EECR |= (1<<EEWE);
  }
  EnableAll();
  return 0;
}
