/*
 *	File:		functions.c
 *  Project:	Oktopod_Board firmware (MCU-AtMega16)
 *	Version:	V3.2
 *  Authors:	Robert Kovacs, Milan Romic
 *	Contact:	robikovacs@oktopodstudio.com, milan.romic@oktopodstudio.com
 *	Web:		<http://www.oktopodstudio.com>

 *	This program is distributed under the terms of the GNU General Public License
 *	Copyright (c) 2010-2013 Oktopod Studio
 *	All rights reserved.
 
 *	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

 *	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 *	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <util/delay.h>
#include "functions.h"
#include "stdlib.h"
#include "globals.h"

volatile signed int takt_prog = 0;

volatile unsigned int
address,
send_timer = 0,
safety_timer = 0,
mode_button_timer = 0,
pwm_timer,
servo_cnt,
led_cnt,
led_pwm,
delay_time_sub_0 = 0,
delay_time_sub_1 = 0,
delay_time_sub_2 = 0,
i;

volatile uchar
standby = 0,
led_puls,
program_block[3],
write_prog,
sub_prog_0_length,
sub_prog_1_length,
sub_prog_2_length,
sub_prog_0_started,
sub_prog_1_started,
sub_prog_2_started,
sub_prog_0_counter,
sub_prog_1_counter,
sub_prog_2_counter,
delay_time_sub_0_flag,
delay_time_sub_1_flag,
delay_time_sub_2_flag,
send[25], 
ch[24],
pwr[15],
pwr_cnt = 1,
ch_cnt,
check,
chc,
message_received = 0,
servo1,
servo2,
servo3,
servo1_tmp,
servo2_tmp,
servo3_tmp;


void InitMCU(void)
{
  DesableAll(); //disable all interrupts
  
	//Port initialization
  	DDRA=0b11110111; //servo1, servo2, servo3, MODE, iz1, iz2, iz3, iz4
	DDRB=0b00000011; //run, conn, input1, input2 
	DDRC=0b11101111; //drajver4, drajver3, drajver2, drajver1, X, in7, iz6, iz5
	DDRD=0b10110010; //RX, TX, input3, input4, pwm2, pwm1, X, iz8

	PORTA=0b00001000;	//pull MODE
	PORTB=0b00001100;	//pull input1, input2 
	PORTC=0b00000000;
	PORTD=0b00001100;	//pull input3, input4 
	
	// USART initialization 	(za 14.7456MHz - no parity, double speed)
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud Rate: 115200 (Double Speed Mode)
// 	UCSRA=0x02;
// 	UCSRB=0xD8;
// 	UCSRC=0x86;
// 	UBRRH=0x00;
// 	UBRRL=0x0F;
	
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud Rate: 57600
// 	UCSRA=0x00;
// 	UCSRB=0xD8;
// 	UCSRC=0x86;
// 	UBRRH=0x00;
// 	UBRRL=0x0F;
	
	//Timer Counter 0 - communication
	TCNT0 = 0x00; 
	TCCR0 = 0b00000101; //presc 128
	TIMSK |= 1 << TOIE0;

	// Timer/Counter 1 initialization - DC MOTOR
	// Clock source: System Clock
	// Clock value: 625.000 kHz
	// Mode: Ph. & fr. cor. PWM top=ICR1
	// OC1A output: Non-Inv.
	// OC1B output: Non-Inv.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer1 Overflow Interrupt: Off
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TCCR1A=0xA0;
	TCCR1B=0x13;
	TCNT1H=0x00;
	TCNT1L=0x00;
	ICR1H=0x00;
	ICR1L=127;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;

	// Timer/Counter 2 initialization - SERVO
	// Clock source: System Clock
	// Clock value: 14745.600 kHz
	// Mode: CTC top=OCR2
	// OC2 output: Disconnected
	ASSR = 0x00;
	TCCR2 = 0b00001010;
	TIMSK |= 1 << OCIE2;	//compare match
	TCNT2 = 0x00;
	OCR2 = 20;
	
	EnableAll();
}

void Bluetooth_setup(void)
{
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	//	USART 9600
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud Rate: 9600
	UCSRA=0x00;
	UCSRB=0xD8;
	UCSRC=0x86;
	UBRRH=0x00;
	UBRRL=0x5F;	

	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	char BAUD7[15] = "AT+BAUD7";	//57600
	Send_USART_data_block(8,BAUD7);
	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
	_delay_ms(80);
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led

	// 	USART 57600
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: On
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud Rate: 57600
	UCSRA=0x00;
	UCSRB=0xD8;
	UCSRC=0x86;
	UBRRH=0x00;
	UBRRL=0x0F;
		
	int  broj, broj1, broj10, broj100, broj1000;
	broj = address;
	broj1 = broj % 10 | 0x30;	
	broj = broj / 10;
	broj10 = broj % 10 | 0x30;
	broj = broj / 10;	
	broj100 = broj % 10 | 0x30;
	broj = broj / 10;
	broj1000 = broj % 10 | 0x30;
		
	char NAME[21] = "AT+NAMEOktopod_" ;	//write BTmodule name
	NAME[15] = broj1000;
	NAME[16] = broj100;
	NAME[17] = broj10;
	NAME[18] = broj1;
	Send_USART_data_block(19,NAME);
	_delay_ms(100);
		
// 	char BAUD4[15] = "AT+BAUD4";	//9600
// 	Send_USART_data_block(8,BAUD4);
// 	_delay_ms(300);
		
	(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
return;
}

void Write_prog_to_EEPROM(void)
{	
	if (ch[2] == 0xFF) return; //Start PROGRAM_MODE
	
	char program_block_to_write[12];
	for (i=0;i<12;i++) 
	{
		program_block_to_write[i] = ch[i+3];	
	}
	if (ch[2] <= 41)	//MAX lenght of the program
	{		
		EEPROM_write_block((int)ch[2] * 12 + 9, program_block_to_write, 12); //write prog from the address 9
	}	
	
	(((PINB &0x01) == 0x01) ? (PORTB &= 0b11111110):(PORTB |= 0b00000001));	//toggle STATUS led
	return;
}

char Execute_prog_line(unsigned char prog_line[3], char pretak)
{
	switch (prog_line[0])
	{
		case 1:	//analog out
		switch(prog_line[1])
		{
			case 1:	//out 1
			pwr[0] = prog_line[2] >> 1;
			return 1;
			break;
			case 2:
			pwr[1] = prog_line[2] >> 1;
			return 1;
			break;
			case 3:
			pwr[2] = prog_line[2] >> 1;
			return 1;
			break;
			case 4:
			pwr[3] = prog_line[2] >> 1;
			return 1;
			break;
			case 5:
			pwr[4] = prog_line[2] >> 1;
			return 1;
			break;
			case 6:
			pwr[5] = prog_line[2] >> 1;
			return 1;
			break;
			case 7:
			pwr[6] = prog_line[2] >> 1;
			return 1;
			break;
			case 8:
			pwr[7] = prog_line[2] >> 1;
			return 1;
			break;
		}
		break;
		
		//DC motor
		case 2:
		switch(prog_line[1])
		{	
			case 9: //motor 1
			if (prog_line[2] > 128)
			{
				SBIT(PORTC, 3);
				CBIT(PORTC, 2);
				OCR1AL = prog_line[2] - 128;
				return 1;
				break;
			}
			else if(prog_line[2] < 128)
			{
				CBIT(PORTC, 3);
				SBIT(PORTC, 2);
				OCR1AL = (128 - prog_line[2]);
				return 1;
				break;
			}
			else
				OCR1AL = 0;
				
			return 1;
			break;
			
			case 10:	//motor 2
			if (prog_line[2] > 128)
			{
				SBIT(PORTC, 1);
				CBIT(PORTC, 0);
				OCR1BL = prog_line[2] - 128;
				
				return 1;
				break;
			}
			else if (prog_line[2] < 128)
			{
				CBIT(PORTC, 1);
				SBIT(PORTC, 0);
				OCR1BL = (128 - prog_line[2]);
				return 1;
				break;
			}
			else
			{
				OCR1BL = 0;
			}				
				
				
			return 1;
			break;
		}
		break;
		
		//servo
		case 3:	
		switch (prog_line[1])
		{
			case 11: //servo 1
			servo1 = (int)((prog_line[2] * 100) / 175) + 60;	//PWM servo 1	scale[60 - 210];
			return 1;
			break;
			case 12: //servo 2
			servo2 = (int)((prog_line[2] * 100) / 175) + 60;
			return 1;
			break;
			case 13: //servo 3
			servo3 = (int)((prog_line[2] * 100) / 175) + 60;
			return 1;
			break;
		}
		break;
		
		//delay
		case 4:
			switch(pretak)
			{
				case 1:
					if(delay_time_sub_0 <= 0 && delay_time_sub_0_flag == 0)
					{
						delay_time_sub_0 =  prog_line[2] * 6;	//scale time 0.1s
						delay_time_sub_0_flag = 1;
						return 0;
					}
					else
					{
						if(delay_time_sub_0 <= 0)
						{
							delay_time_sub_0_flag = 0;
							return 1;
						}
						else
							return 0;
					}
				break;
				case 2:
					if(delay_time_sub_1 <= 0 && delay_time_sub_1_flag == 0)
					{
						delay_time_sub_1 = prog_line[2] * 6;	//scale time 0.1s
						delay_time_sub_1_flag = 1;
						return 0;
					}
					else
					{
						if(delay_time_sub_1 <= 0)
						{
							delay_time_sub_1_flag = 0;
							return 1;
						}
						else
							return 0;
					}
				break;
				case 3:
					if(delay_time_sub_2 <= 0 && delay_time_sub_2_flag == 0)
					{
						delay_time_sub_2 = prog_line[2] * 6;	//scale time 0.1s
						delay_time_sub_2_flag = 1;
					}
					else
					{
						if(delay_time_sub_2 <= 0)
						{
							delay_time_sub_2_flag = 0;
							return 1;
						}
						else
							return 0;
					}
				break;
			}									
		break;
		
		//wait until
		case 5:
		switch (prog_line[1])
		{
			case 1:	//PORTB2
			if (((PINB & 0x04) >> 2) == prog_line[2])	
				return 0;
			else
				return 1;
			case 2: //PORTB3
			if (((PINB & 0x08) >> 3) == prog_line[2])
				return 0;
			else
				return 1;
			case 3: //PORTD2
			if (((PIND & 0x04) >> 2) == prog_line[2])
				return 0;
			else
				return 1;
			case 4: //PORTD2
			if (((PIND & 0x08) >> 3) == prog_line[2])
				return 0;
			else
				return 1;
		}
		break;
		
		//reset all
		case 98:
			Reset_outputs();
			return 1;
		break;
		
		//Start - stop prog
		case 99:
		switch (prog_line[1])
		{
			case 1:
				sub_prog_0_started = 1;
				return 1;
			break;
			case 2:
				sub_prog_1_started	= 1;
				return 1;
			break;
			case 3:
				sub_prog_2_started = 1;
				return 1;
			break;
			case 4:
				sub_prog_0_started = 0;
				sub_prog_0_counter = 0;	
				return 1;
			break;
			case 5:
				sub_prog_1_started	= 0;
				sub_prog_1_counter = 0;
				return 1;
			break;
			case 6:
				sub_prog_2_started = 0;
				sub_prog_2_counter = 0;
				return 1;
			break;
		}
		break;
	}
return 0;
}

void Analog_output_PWM()
{
	//Analog Out PWM
	if(pwr_cnt == 1)
	{
		pwr_cnt = 128;
		PORTA &= 0b00001111;
		PORTC &= 0b00011111;
		PORTD &= 0b01111111;
	}
	pwr_cnt--;
	if(pwr_cnt == pwr[0]) SBIT(PORTA, 4);
	if(pwr_cnt == pwr[1]) SBIT(PORTA, 5);
	if(pwr_cnt == pwr[2]) SBIT(PORTA, 6);
	if(pwr_cnt == pwr[3]) SBIT(PORTA, 7);
	if(pwr_cnt == pwr[4]) SBIT(PORTC, 7);
	if(pwr_cnt == pwr[5]) SBIT(PORTC, 6);
	if(pwr_cnt == pwr[6]) SBIT(PORTC, 5);
	if(pwr_cnt == pwr[7]) SBIT(PORTD, 7);
}

void Communication_timeout_output_reset()
{
	//reset all outputs if communication stops
	if (safety_timer > 50)
	{
		standby = 1;
		Reset_outputs();
	}
}
void Reset_outputs(void)
{
	//reset  outputs
	for (i=0;i<8;i++) pwr[i] = 0;	//reset outputs
	PORTA &= 0b00001111;
	PORTC &= 0b00011111;
	PORTD &= 0b01111111;
	OCR1BL = 0;			//dc motor1 stop
	OCR1AL = 0;			//dc motor2 stop
	servo1 = 131;		//servo1 middle
	servo2 = 131;		//servo2 middle
	servo3 = 131;		//servo3 middle
}

void Send_data(void) //addr1, addr2, input1, input2, input3, input4, CHC (all 6 byte)
{
	send[0] = address >> 8;
	send[1] = address;	
		
	(((PINB & 0x04) == 0x04) ? (send[2] = 0):(send[2] = 1));	//input1
	(((PINB & 0x08) == 0x08) ? (send[3] = 0):(send[3] = 1));	//input2
	(((PIND & 0x04) == 0x04) ? (send[4] = 0):(send[4] = 1));	//input3
	(((PIND & 0x08) == 0x08) ? (send[5] = 0):(send[5] = 1));	//input4
	
	check=0;	//calculate CHC
	for (i=0; i<=5; i++) 		
		check=check+send[i];	

	send[6]=check;

	Send_USART_data_block(7, send);
}

void Receive_data()
{
	standby = 0;
	safety_timer = 0;
	message_received = 0;
	
	//set analog outputs
	for (i=2; i<=9; i++)
	{
		if (ch[i] != 0xFF)	//0xFF no action	
			pwr[i-2]=ch[i] >> 1;	
	}
					
	//PWM mot1
	if (ch[10] == 0xFF);	//0xFF no action
	else if (ch[10] > 128)
	{
		SBIT(PORTC, 3);
		CBIT(PORTC, 2);
		OCR1AL = ch[10] - 128;
	}
	else if(ch[10] < 128)
	{
		CBIT(PORTC, 3);
		SBIT(PORTC, 2);
		OCR1AL = (128 - ch[10]);
	}
	else
		OCR1AL = 0;
			
	//PWM mot2	
	if (ch[11] == 0xFF);	//0xFF no action
	else if (ch[11] > 128)
	{
		SBIT(PORTC, 1);
		CBIT(PORTC, 0);
		OCR1BL=ch[11] - 128;
	}
	else if (ch[11] < 128)
	{
		CBIT(PORTC, 1);
		SBIT(PORTC, 0);
		OCR1BL=(128 - ch[11]);
	}
	else
		OCR1BL = 0;
		
	//servo motor (70 - 190)
	if (ch[12] != 0xFF)	//0xFF no action	
	{	
		servo1 = (int)((ch[12] * 100) / 175) + 60;	//PWM servo 1	[60 - 200]
	}	
	if (ch[13] != 0xFF)	//0xFF no action	
	{
		servo2 = (int)((ch[13] * 100) / 175) + 60;	//PWM servo 2	[60 - 200]
	}		
	if (ch[14] != 0xFF)	//0xFF no action	
	{
		servo3 = (int)((ch[14] * 100) / 175) + 60;	//PWM servo 3	[60 - 200]	
	}			
}