/*
 *	File:		interrupt.c
 *  Project:	Oktopod_Board Firmware (MCU-AtMega16)
 *	Version:	V3.2
 *  Authors:	Robert Kovacs, Milan Romic
 *	Contact:	robikovacs@oktopodstudio.com, milan.romic@oktopodstudio.com
 *	Web:		<http://www.oktopodstudio.com>
 
  *	This program is distributed under the terms of the GNU General Public License
 *	Copyright (c) 2010-2013 Oktopod Studio
 *	All rights reserved.
 
 *	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

 *	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 *	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "globals.h"
#include "communication.h"
#include "functions.h"

//servo timer
ISR (TIMER2_COMP_vect)	
{
	//SATUS led in pulsing mode	
	if(standby == 1) 
	{
		led_cnt++;
		if (led_cnt > (led_pwm >> 8))
		{
			CBIT(PORTB, 1); //STATUS led on
		}
		if (led_cnt >= 200)
		{
			led_cnt = 0;
			SBIT(PORTB, 1); //STATUS led off
		}
	
		if (led_puls == 0)
		{	
			led_pwm++;
			if (led_pwm > 55000)
				led_puls = 1;
		}
		else
		{
			led_pwm--;
			if (led_pwm < 2500)
				led_puls = 0;
		}
	}	
	else //servo PWM
	{
		if(servo_cnt > 2000 - servo1) SBIT(PORTA,0); //servo 1
		if(servo_cnt > 2000 - servo2) SBIT(PORTA,1); //servo 2
		if(servo_cnt > 2000 - servo3) SBIT(PORTA,2); //servo 3
		if (servo_cnt >= 2000)
		{
			servo_cnt = 0;	//cycle 20ms
			PORTA &= 0b11111000;
		}
		servo_cnt++; 
		
		
		//analog outputs
		pwm_timer++;
		if (pwm_timer == 5)
		{
			pwm_timer = 0;
			//Analog Out PWM
			if(pwr_cnt == 1)
			{
				pwr_cnt = 128;
				PORTA &= 0b00001111;
				PORTC &= 0b00011111;
				PORTD &= 0b01111111;
			}
			pwr_cnt--;
			if(pwr_cnt == pwr[0]) SBIT(PORTA, 4);
			if(pwr_cnt == pwr[1]) SBIT(PORTA, 5);
			if(pwr_cnt == pwr[2]) SBIT(PORTA, 6);
			if(pwr_cnt == pwr[3]) SBIT(PORTA, 7);
			if(pwr_cnt == pwr[4]) SBIT(PORTC, 7);
			if(pwr_cnt == pwr[5]) SBIT(PORTC, 6);
			if(pwr_cnt == pwr[6]) SBIT(PORTC, 5);
			if(pwr_cnt == pwr[7]) SBIT(PORTD, 7);
		}
	}	
	
		
return; 
}

//main program timer
ISR (TIMER0_OVF_vect)
{	
	//send data timer
	send_timer++;
	//safety reset timer
	safety_timer++;
	//program execution timer
	takt_prog++;
	//program delay time
	if (delay_time_sub_0 > 0 ) delay_time_sub_0--;
	if (delay_time_sub_1 > 0 ) delay_time_sub_1--;
	if (delay_time_sub_2 > 0 ) delay_time_sub_2--;
	//mode button timer
	if (mode_button_timer > 0) mode_button_timer--;
	return;
}

//USART data received interrupt
ISR (USART_RXC_vect)
{
 	ch[ch_cnt] = UDR;
	if (ch_cnt <= 14) chc += ch[ch_cnt];
	ch_cnt++;

	if(ch[0] == 0x30 || ch[0] == 0x55)	//START BYTE or boot loader reset (UUUUUUUU...)
	{
		if (ch_cnt >= 16)	//complete message arrived
		{
			if(ch[15] == chc)	//CHC check
			{
				//command for remote control															 
				if (ch[1] == 1) //format (16 bytes) >> addr0, addr1, out0, out1...out7(0-255), mot1, mot2(R:0-128,L:128-256), ser1, ser2, ser3, CHC(mod255(byte1+byte2+...+byte15))
				{
					message_received = 1;
				}	
				//command for program download to EEPROM
				else if (ch[1] == 2) 	//format  [0x30, 0x02, redni_br, duzina_prog1, duzina_prog2, duzina_prog3, lin01, lin02, lin03, lin11, lin12, lin13, lin21, lin22, lin23,(lin31, lin32, lin33,) CHC]
				{																 
					write_prog = 1;													  
				}	
				//command for address update
				else if (ch[1] == 3) 	  //format [0x30, 0x03, address_HI, address_LO, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, CHC]
				{																 
					EEPROM_write_char(510,	ch[2]);	//address_HI
					EEPROM_write_char(511,	ch[3]);	//sddress_LO	
					Send_USART_data_block(2, "OK");	
					goto *0x0000; 									  
				}			
				ch_cnt = 0;
				chc = 0;
				(((PINB & 0x02) == 0x02) ? (CBIT(PORTB, 1)):(SBIT(PORTB, 1)));	//toggle STATUS led
			}	
			else if (chc == 0xFB)	//UUUUUUUUUUUUUUU... - GOTO BOOTLOADER
			{
				SBIT(PORTB, 1);	//toggle STATUS led
				wdt_enable(WDTO_15MS);
			}
			else
			{
				ch_cnt=0;
				chc = 0;			
			}	
		}			
	}
	else //wrong address
	{
		ch_cnt = 0;
		chc = 0;
	}
  return;
}

ISR (USART_UDRE_vect)
{
	return;
}

ISR (USART_TXC_vect)
{
	return;
}
ISR (INT0_vect)
{
	return;
}

ISR (INT1_vect)
{
	return;
}
ISR (TIMER2_OVF_vect)
{
	return;
}

ISR (TIMER1_CAPT_vect)
{
	return;
}

ISR (TIMER1_COMPA_vect)
{
	
	return;
}

ISR (TIMER1_COMPB_vect)
{
	return;
}

ISR (TIMER1_OVF_vect)
{
	return;
}

ISR (ADC_vect)
{
	return;
}
    
ISR (EE_RDY_vect)
{
	return;
}

ISR (ANA_COMP_vect)
{
	return;
}

ISR (TWI_vect)
{            
  return;
}

ISR (INT2_vect)
{
	return;
}

ISR (TIMER0_COMP_vect)
{

	return;
}

ISR (SPM_RDY_vect)
{
	return;
}

