/*
 *	File:		oktopod_main.c
 *  Project:	Oktopod_Board firmware (MCU-AtMega16)
 *	Version:	V3.2
 *  Authors:	Robert Kovacs, Milan Romic
 *	Contact:	robikovacs@oktopodstudio.com, milan.romic@oktopodstudio.com
 *	Web:		<http://www.oktopodstudio.com>
 
  *	This program is distributed under the terms of the GNU General Public License
 *	Copyright (c) 2010-2013 Oktopod Studio
 *	All rights reserved.
 
 *	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

 *	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 *	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include "delay.h"
#include "globals.h"
#include "communication.h"
#include "functions.h"
#include "stdlib.h"
#include "eeprom.h"


int main(void) 
{	
	wdt_disable();
	//read interface address
	uchar addressHI, addressLO;
	EEPROM_read_char(510,&addressHI);
	EEPROM_read_char(511,&addressLO);
	address = (int)(addressHI << 8) + addressLO;
	
	InitMCU();
	Bluetooth_setup();
	mode_button_timer = 255;

	//////////////////////////////////REMOTE MODE/////////////////////////////////////////////
	REMOTE_MODE:
	SBIT(PORTB, 0); //Satus LED green
	Reset_outputs();
	mode_button_timer = 100;
	
	while(1)	//main loop
	{		
		//if new command arrives
		if(message_received == 1)
		{
			Receive_data();
		}		
		Communication_timeout_output_reset();
		
		//if new program downloaded run the program immediately
		if (write_prog == 1)
		{
			goto PROGRAM_MODE;
		}	
		
		//Send state
		if(send_timer >= 5)	//~150ms
		{
			Send_data();
			send_timer = 0;
		}	
		
		//if MODE button pressed for 4s > goto RESET
		if ((PINA & 0x08) != 0x08)
		{
			if (mode_button_timer == 0)
			{
				mode_button_timer = 255;
				goto *0x1E00; //BOOT RESET
			}
		}
		else if (mode_button_timer < 95)
		{
			mode_button_timer = 100;
			goto PROGRAM_MODE;
		}
		else
			mode_button_timer = 100;
	}
	
	///////////////////////////////////PROGRAM MODE//////////////////////////////////////////////
	PROGRAM_MODE:
	standby = 0;
	SBIT(PORTB, 1); //Status LED red
	//reset state
	sub_prog_0_started = 1;
	sub_prog_1_started = 1;
	sub_prog_2_started = 1;
	sub_prog_0_counter = 0;	
	sub_prog_1_counter = 0;
	sub_prog_2_counter = 0;
	Reset_outputs();
	//reed program lenght
	EEPROM_read_char(9, &sub_prog_1_length);	 
	EEPROM_read_char(10, &sub_prog_1_length);	
	EEPROM_read_char(11, &sub_prog_2_length);	
	
	mode_button_timer = 100;
	
	while(1)
	{
		//If new program received
		if (write_prog == 1)		//write to  EEPROM
		{
			write_prog = 0;
			Write_prog_to_EEPROM();
			takt_prog = -50;		//little delay before beginning
			goto PROGRAM_MODE;		//start
		}
		
		//send state
		if(send_timer >= 5)	//~150ms
		{
			Send_data();
			send_timer = 0;
		}	
		
		//if message received from control device switch to REMOTE_MODE
		if(message_received == 1)
		{
			goto REMOTE_MODE;
		}
		
		//if MODE for 4s > goto RESET
		if ((PINA & 0x08) != 0x08)
		{
			if (mode_button_timer == 0)
			{
				mode_button_timer = 255;
				goto *0x1E00; //BOOT RESET
			}			
		}
		else if (mode_button_timer < 95)
		{
			mode_button_timer = 100;
			goto REMOTE_MODE;
		}
		else
			mode_button_timer = 100;
					
		
		//EXECUTE THE PROGRAM
		if(takt_prog >= 1)	//frequency of the execution
		{	
			takt_prog = 0;
			//SUB PROG 0
			if(sub_prog_0_started == 1 && sub_prog_1_length > 0)
			{
				if (sub_prog_0_counter >= sub_prog_0_length)	sub_prog_0_counter = 0;	//end of sub prog	
					
				EEPROM_read_block((int)(sub_prog_0_counter * 3) + 12, &program_block, 3);	//read prog line

				//execute
				if((Execute_prog_line(program_block, 1)) == 1)
					sub_prog_0_counter++;
			}			
		
			//SUB PROG 1
			if(sub_prog_1_started == 1 && sub_prog_1_length > 0)	
			{
				if (sub_prog_1_counter >= sub_prog_1_length)	sub_prog_1_counter = 0;	//end of sub prog
					
				EEPROM_read_block((int)(sub_prog_1_counter * 3) + 12 + sub_prog_1_length * 3, &program_block, 3);	//read prog line
				
				//execute
				if(Execute_prog_line(program_block, 2) == 1)
					sub_prog_1_counter++;
			}
		
			//SUB PROG 2
			if(sub_prog_2_started == 1 && sub_prog_2_length > 0)
			{
				if (sub_prog_2_counter >= sub_prog_2_length) sub_prog_2_counter = 0;	//end of sub prog
						
				EEPROM_read_block((int)(sub_prog_2_counter * 3) + 12 + (sub_prog_1_length + sub_prog_1_length) * 3, &program_block, 3);	//read prog line

				//execute
				if(Execute_prog_line(program_block, 3) == 1)	
					sub_prog_2_counter++;
			}
			
			(((PINB &0x01) == 0x01) ? (CBIT(PORTB, 0)):(SBIT(PORTB, 0)));	//togle RUN led
		} 
	}
}
